@@ -3,13 +3,15 @@
 """
 __author__ = "Axelle Apvrille"
 __license__ = "MIT License"
-__version__ = '3.2.1'
+__version__ = '3.4.0'
 """
 import hashlib
+import base64
 import os
 import re
 import itertools
 import sys
+import string
 import struct
 import zlib
 import shutil
@@ -216,79 +218,108 @@ class droidsample:
                 subprocess.call(["java", "-jar", droidconfig.APKTOOL_JAR, \
                                  "-q", "d", "-f", self.absolute_filename,  \
                                  "-o", apktool_outdir ], stdout=self.process_output, stderr=self.process_output)
-                
-            if self.verbose:
-                print( "Apktool finished" )
 
             if os.path.isdir(apktool_outdir):
                 droidutil.move_dir(apktool_outdir, self.outdir)
 
+            # we want all smali_classes? dir in smali
+            if os.path.exists(os.path.join(self.outdir, "smali_classes2")):
+                # we have multidex - we are going to move all smali classes in the same directory
+                if self.verbose:
+                    print("Moving multidex smali classes to ./smali")
+
+                os.system("cp -R "+ os.path.join(self.outdir, "./smali_classes?/*") + " " + os.path.join(self.outdir, "./smali") )
+                os.system("rm -r "+ os.path.join(self.outdir, "./smali_classes?") )
+                
+            if self.verbose:
+                print( "Apktool finished" )
+
             # extract classes.dex whatever happens, we'll use it
             if self.verbose:
-                print( "Extracting classes.dex" )
+                print( "Extracting classes*.dex" )
             try:
                 self.ziprar.extract_one_file('classes.dex', self.outdir)
+                for i in range(2,10):
+                    self.ziprar.extract_one_file('classes{}.dex'.format(i), self.outdir)
+                    if not os.path.exists(os.path.join(self.outdir, 'classes{}.dex'.format(i))):
+                        break
+                    self.properties.smali['multidex'].append('classes{}.dex'.format(i))
+                    
             except:
                 if self.verbose:
                     print( "Extracting classes.dex failed: %s" % (sys.exc_info()[0]))
                 else:
                     print( "Extracting classes.dex failed")
 
-        # Disassemble the DEX
+        # Disassemble the DEX(es)
+        dex_files = []
         if self.properties.filetype == droidutil.DEX:
-            dex_file = self.absolute_filename
+            dex_files.append( self.absolute_filename )
         else:
-            dex_file = os.path.join(self.outdir, 'classes.dex')
+            if len(self.properties.smali['multidex']) > 0:
+                for d in self.properties.smali['multidex']:
+                    dex_files.append(os.path.join(self.outdir, d))
+            else:
+                dex_files.append(os.path.join(self.outdir, 'classes.dex'))
 
         smali_dir = os.path.join( self.outdir, 'smali' )
 
         if (not os.access( smali_dir, os.R_OK) or \
-                (os.access(smali_dir, os.R_OK) and not os.listdir(smali_dir)) \
-                and os.access( dex_file, os.R_OK)):
-            if self.verbose:
-                print( "Using baksmali on " + dex_file )
-            try:
-                subprocess.call( [ "java", "-jar", droidconfig.BAKSMALI_JAR, \
-                                   "d", "-o", smali_dir, dex_file ], \
-                                 stdout=self.process_output, stderr=self.process_output)
-            except:
-                print( "Baksmali failed" )
+                (os.access(smali_dir, os.R_OK) and not os.listdir(smali_dir))):
+            # we only do this if apktool didn't baksmali correctly
+            for d in dex_files:
+                if os.access(d, os.R_OK):
+                    if self.verbose:
+                        print( "Baksmali on {} -> {}".format(d, smali_dir))
+                    try:
+                        subprocess.call( [ "java", "-jar", droidconfig.BAKSMALI_JAR, \
+                                           "d", "-o", smali_dir, d ], \
+                                         stdout=self.process_output, stderr=self.process_output)
+                    except:
+                        print( "[-] Baksmali failed for {}".format(d) )
 
-        # Decompile the DEX
+        # Decompile the DEX(es)
         if self.verbose:
             print("------------- Decompiling")
-            
-        if not self.clear and os.access( dex_file, os.R_OK ):
-            jar_file = os.path.join(self.outdir, 'classes-dex2jar.jar')
-            if os.access( droidconfig.DEX2JAR_CMD, os.X_OK):
-                if self.verbose:
-                    print( "Dex2jar on " + dex_file )
-                subprocess.call( [ droidconfig.DEX2JAR_CMD, "--force", dex_file, "-o", jar_file ], \
-                                 stdout=self.process_output, stderr=self.process_output)
-            else:
-                if self.verbose:
-                    print("Dex2jar: file is not executable, skipping (file: {0})".format(droidconfig.DEX2JAR_CMD))
+
+        if not self.clear:
+            for d in dex_files:
+                if os.access(d, os.R_OK):
+                    jar_file = os.path.join(self.outdir, '{}-dex2jar.jar'.format(os.path.splitext(os.path.basename(d))[0]))
+                    if os.access( droidconfig.DEX2JAR_CMD, os.X_OK):
+                        if self.verbose:
+                            print( "Dex2jar on " + d )
+                            try:
+                                subprocess.call( [ droidconfig.DEX2JAR_CMD, "--force", d, "-o", jar_file ], \
+                                                 stdout=self.process_output, stderr=self.process_output)
+                            except:
+                                if self.verbose:
+                                    print("[-] Dex2jar failed on {}".format(d))
+                        else:
+                            if self.verbose:
+                                print("Dex2jar software is not executable, skipping (file: {0})".format(droidconfig.DEX2JAR_CMD))
                     
-            if os.access( jar_file, os.R_OK ):
-                if self.enable_procyon:
+                    if os.access( jar_file, os.R_OK ):
+                        if self.enable_procyon and os.access( droidconfig.PROCYON_JAR, os.R_OK ):
+                            if self.verbose:
+                                print( "Procyon decompiler on " + jar_file )
+                            subprocess.call( [ "java", "-jar", droidconfig.PROCYON_JAR, \
+                                               jar_file, "-o", os.path.join(self.outdir, 'procyon') ], \
+                                             stdout=self.process_output, stderr=self.process_output)
+
                     if self.verbose:
-                        print( "Procyon decompiler on " + jar_file )
-                    subprocess.call( [ "java", "-jar", droidconfig.PROCYON_JAR, \
-                                           jar_file, "-o", os.path.join(self.outdir, 'procyon') ], \
-                                         stdout=self.process_output, stderr=self.process_output)
-                if self.verbose:
-                    print( "Unjarring " + jar_file )
-                jarziprar = droidziprar.droidziprar(jar_file, zipmode=True, verbose=self.verbose)
-                if jarziprar.handle == None:
-                    if self.verbose:
-                        print( "Bad Jar / Failed to unjar " + jar_file )
-                else:
-                    try:
-                        jarziprar.extract_all(os.path.join(self.outdir, 'unjarred'))
-                    except:
-                        print( "Failed to unjar: %s" % (sys.exc_info()[0]) )
-                    jarziprar.close()
-                
+                        print( "Unjarring " + jar_file )
+                    jarziprar = droidziprar.droidziprar(jar_file, zipmode=True, verbose=self.verbose)
+                    if jarziprar.handle == None:
+                        if self.verbose:
+                            print( "Bad Jar / Failed to unjar " + jar_file )
+                    else:
+                        try:
+                            jarziprar.extract_all(os.path.join(self.outdir, 'unjarred'))
+                        except:
+                            print( "Failed to unjar: %s" % (sys.exc_info()[0]) )
+                        jarziprar.close()
+
         # Convert binary Manifest
         if self.properties.filetype == droidutil.APK:
             manifest = os.path.join( self.outdir, 'AndroidManifest.xml')
@@ -448,11 +479,26 @@ class droidsample:
                     print( "Certificate owner matches Famous : "+m.group(0) )
                 self.certificate['famous'] = True
 
-            
+    def is_packed(self):
+        """
+        We assume a sample is packed if the main activity its manifest references cannot be found in the DEX + DexClassLoader or Dex File
+        This test is far from perfect
+        """
+        smali_dir = os.path.join(self.outdir, 'smali')
+        missing = False
+        if self.properties.manifest['main_activity'] != None:
+            filename = os.path.join(smali_dir, self.properties.manifest['main_activity'].replace('.', os.path.sep).replace('\'','') + '.smali')
+            if not os.access(filename, os.R_OK):
+                if self.verbose:
+                    print("Unable to find Main Activitity: {} filename={}".format(self.properties.manifest['main_activity'],filename))
+                missing = True
 
+        if missing and (self.properties.smali['dex_class_loader'] or self.properties.smali['dex_file']):
+            self.properties.smali['packed'] = True
+                
 
     def extract_manifest_properties(self):
-        """Extracting services, receviers, activities etc from manifest"""
+        """Extracting services, receivers, activities etc from manifest"""
         manifest = os.path.join(self.outdir, 'AndroidManifest.xml')
         if self.properties.filetype == droidutil.APK and os.access(manifest, os.R_OK) and os.path.getsize(manifest)>0:
             try:
@@ -462,29 +508,55 @@ class droidsample:
                     print( "XML parsing error" )
                 return;
 
+            # getting package's name
+            self.properties.manifest['package_name'] = xmldoc.documentElement.getAttribute('package')
+
+
             tab = droidutil.get_elements(xmldoc, 'service', 'android:name')
+            tab.extend(droidutil.get_elements(xmldoc, 'service', 'obfuscation:name'))
             for t in tab:
-                self.properties.manifest['services'].append(re.sub(r"u'(?P<name>.*)'", r"\g<name>", t))
+                name = re.sub(r"u'(?P<name>.*)'", r"\g<name>", t)
+                if name != "''":
+                    self.properties.manifest['services'].append(name)
                                                             
             tab = droidutil.get_elements(xmldoc, 'receiver', 'android:name')
+            tab.extend(droidutil.get_elements(xmldoc, 'receiver', 'obfuscation:name'))
             for t in tab:
-                self.properties.manifest['receivers'].append(re.sub(r"u'(?P<nom>.*)'", r"\g<nom>", t))
+                name = re.sub(r"u'(?P<name>.*)'", r"\g<name>", t)
+                if name != "''" :
+                    self.properties.manifest['receivers'].append(name)
 
             tab = droidutil.get_elements(xmldoc, 'activity', 'android:name')
+            tab.extend(droidutil.get_elements(xmldoc, 'activity', 'obfuscation:name'))
             for t in tab:
-                self.properties.manifest['activities'].append(re.sub(r"u'(?P<nom>.*)'", r"\g<nom>", t))
+                name = re.sub(r"u'(?P<name>.*)'", r"\g<name>", t)
+                if name != "''" :
+                    self.properties.manifest['activities'].append(name)
 
             tab = droidutil.get_elements(xmldoc, 'provider', 'android:name')
+            tab.extend(droidutil.get_elements(xmldoc, 'provider', 'obfuscation:name'))
             for t in tab:
-                self.properties.manifest['providers'].append(re.sub(r"u'(?P<nom>.*)'", r"\g<nom>", t))
+                name = re.sub(r"u'(?P<name>.*)'", r"\g<name>", t)
+                if name != "''" :
+                    self.properties.manifest['providers'].append(name)
 
             tab = droidutil.get_elements(xmldoc, 'uses-library', 'android:name')
             for t in tab:
                 self.properties.manifest['libraries'].append(re.sub(r"u'(?P<nom>.*)'", r"\g<nom>", t))
 
             tab = droidutil.get_elements(xmldoc, 'uses-permission', 'android:name')
+            tab.extend(droidutil.get_elements(xmldoc, 'uses-permission', 'obfuscation:name'))
             for t in tab:
-                self.properties.manifest['permissions'].append(t.replace('android.permission.','').replace("u'", '').replace("'", ''))
+                name = t.replace('android.permission.','').replace("u'", '').replace("'", '')
+                if name != '':
+                    self.properties.manifest['permissions'].append(name)
+
+            tab = droidutil.get_elements(xmldoc, 'application', 'obfuscation:name')
+            tab.extend(droidutil.get_elements(xmldoc, 'application', 'android:name'))
+            for t in tab:
+                if 'multidex' in t:
+                    self.properties.manifest['multidex'] = True
+                    break
 
             self.properties.manifest['maxSDK'] = droidutil.get_element(xmldoc, 'uses-sdk', 'android:maxSdkVersion')
             self.properties.manifest['minSDK'] = droidutil.get_element(xmldoc, 'uses-sdk', 'android:minSdkVersion')
@@ -499,10 +571,13 @@ class droidsample:
             # get main activity
             for actitem in xmldoc.getElementsByTagName('activity'):
                 for a in actitem.getElementsByTagName('action'):
-                    if a.getAttribute( 'android:name' ) == 'android.intent.action.MAIN':
+                    if a.getAttribute( 'android:name' ) == 'android.intent.action.MAIN' or a.getAttribute('obfuscation:name') ==  'android.intent.action.MAIN':
                         for b in actitem.getElementsByTagName('category'):
-                            if b.getAttribute( 'android:name' ) == 'android.intent.category.LAUNCHER':
-                                self.properties.manifest['main_activity'] = actitem.getAttribute( 'android:name' )
+                            if b.getAttribute( 'android:name' ) == 'android.intent.category.LAUNCHER' or b.getAttribute('obfuscation:name') ==  'android.intent.category.LAUNCHER':
+                                if actitem.getAttribute('android:name') != '':
+                                    self.properties.manifest['main_activity'] = actitem.getAttribute( 'android:name' )
+                                else:
+                                    self.properties.manifest['main_activity'] = actitem.getAttribute( 'obfuscation:name' )                  
             if self.verbose and self.properties.manifest['main_activity'] != None:
                 print( "Main activity: " + self.properties.manifest['main_activity'])
 
@@ -525,11 +600,6 @@ class droidsample:
                 print( "Listens to incoming SMS  : %d" % (self.properties.manifest['listens_incoming_sms']))
                 print( "Listens to outgoing calls: %d" % (self.properties.manifest['listens_outgoing_call']))
                 
-            # get package name
-            if not self.clear:
-                self.properties.manifest_package = droidutil.get_element(xmldoc, 'manifest', 'package')
-                if self.verbose:
-                    print( "Package's name : %s" % (self.properties.manifest_package))
 
     def extract_kit_properties(self):
         """
@@ -543,12 +613,15 @@ class droidsample:
                 for section in self.properties.kitsconfig.get_sections():
                     pattern_list = self.properties.kitsconfig.get_pattern(section).split('|')
                     for pattern in pattern_list:
-                        if os.access(os.path.join(smali_dir, pattern), os.R_OK):
-                            if self.verbose:
-                                print("kits[%s] = True (detected pattern: %s)" % (section, pattern))
-                            list.append(section)
-                            self.properties.kits[ section ] = True
-                            break # break one level
+                        for root, dirs, fname in os.walk(smali_dir):
+                            if pattern in root:
+                                if self.verbose:
+                                    print("kits[%s] = True (detected pattern: %s)" % (section, pattern))
+                                list.append(section)
+                                self.properties.kits[ section ] = True
+                                break # break one level
+                        if self.properties.kits[ section ] == True:
+                            break # break another level
         return list
 
     def extract_dex_properties(self):
@@ -646,13 +719,14 @@ class droidsample:
         if os.access(smali_dir, os.R_OK) and os.listdir(smali_dir) != []: 
             exceptions = []
             for kit in list_of_kits:
-                pattern = self.properties.kitsconfig.get_pattern(kit)
-                if pattern != None and pattern != '':
-                    exceptions.append(os.path.join(smali_dir, pattern ))
-                else:
-                    if self.verbose:
-                        print( "WARNING: configuration file error: empty pattern for %s" % (kit) )
-
+                pattern_list = self.properties.kitsconfig.get_pattern(kit).split('|')
+                for pattern in pattern_list:
+                    if pattern != None and pattern != '':
+                        exceptions.append(pattern) # pattern may be part of a path so do not prefix with smali_dir
+                    else:
+                        if self.verbose:
+                            print( "WARNING: configuration file error: empty pattern for %s" % (kit) )
+                              
             smali_regexp = self.properties.smaliconfig.get_all_regexp()
             match = droidutil.recursive_search(smali_regexp, smali_dir, exceptions, False)
             
@@ -686,7 +760,9 @@ class droidsample:
                         analysis_file.write("- "+str(element)+"\n")
                     analysis_file.write('\n')
                     analysis_file.close()
-                
+
+            # test if sample is likely to be packed
+            self.is_packed()
         else:
             if self.verbose:
                 print( "Cannot extract smali properties, because directory %s not found" % (smali_dir))
@@ -713,12 +789,14 @@ class droidsample:
         if self.properties.filetype == droidutil.APK or self.properties.filetype == droidutil.DEX:
             exceptions = []
             for kit in list_of_kits:
-                exceptions.append(os.path.join(os.path.join(self.outdir, 'smali'), self.properties.kitsconfig.get_pattern(kit)))
+                pattern_list = self.properties.kitsconfig.get_pattern(kit).split('|')
+                for pattern in pattern_list:
+                    exceptions.append(pattern)
             exceptions.append("/unjarred")
             exceptions.append("/unzipped")
             exceptions.append("/unknown")
             exceptions.append("classes.dex")
-            exceptions.append('autoanalysis.md')
+            exceptions.append('details.md')
 
             wide_regexp = self.properties.wideconfig.get_all_regexp()
             match = droidutil.recursive_search(wide_regexp, self.outdir, exceptions, False)
@@ -800,8 +878,8 @@ class droidsample:
             output = proc.communicate()
 
             for section in self.properties.armconfig.get_sections():
-                matches = re.findall(self.properties.armconfig.get_pattern(section), output[0])
-                if matches != None:
+                matches = re.findall(self.properties.armconfig.get_pattern(section), output[0].decode('utf-8'))
+                if matches != None and len(matches) > 0:
                     self.properties.arm[section] = True
                     if self.verbose:
                         print( "Setting arm[%s]  = True" % (section))
@@ -809,7 +887,6 @@ class droidsample:
                         for m in matches:
                             self.interesting_url(m)
 
-
     def find_executables(self, dir):
         """Finds executables in a given directory. Does not parse recursively.
         Returns two lists:
@@ -862,8 +939,9 @@ class droidsample:
         asset_dir = os.path.join(self.outdir, "assets")
         raw_dir = os.path.join(self.outdir, "res/raw")
         lib_dir = os.path.join(self.outdir, "lib/armeabi")
-
-        list_dir = [ asset_dir, raw_dir, lib_dir ]
+        lib2_dir = os.path.join(self.outdir, "lib/arm64-v8a")
+        
+        list_dir = [ asset_dir, raw_dir, lib_dir, lib2_dir ]
 
         for dir in list_dir:
             if os.access(dir, os.R_OK) and os.path.isdir(dir):
@@ -919,42 +997,46 @@ class droidsample:
         return self.properties.wide['urls']
 
     def find_base64_strings(self, thedir, exceptions, theproperty):
+        '''
+        To detect base64 strings, we spot all constants with potentially base64 encoded
+        characters. Then, on these, we attempt to perform base 64 decoding.
+        If that does not lead to an error (padding exception etc) and if the result is
+        printable (~ makes potential sense), we assume this is a Base64 string
+        '''
         # const-string v1, "xyz=="
         # this won't detect all base64 strings because they won't all finish with ==
         # but we'll get some...
-        base64_regexp = bytes("const-string v[0-9]*, \"[a-zA-Z0-9/=]*==\"", 'utf-8')
+        base64_regexp = bytes("const-string v[0-9]*, \"[a-zA-Z0-9/?=]*\"", 'utf-8')
         base64_strings = droidutil.recursive_search(base64_regexp, thedir, exceptions, False)
 
         if base64_strings is not None:
-            analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
-            analysis_file.write("Base64 strings:\n")
-            analysis_file.close()
-            
+            printable_chars = bytes(string.printable, 'ascii')
             for s in base64_strings.keys():
-                # remove the const-string vX part
+                # remove the const-string vX part and trailing "
                 thestr = re.sub('const-string v[0-9]*, "', '', s)
-                thestr = re.sub('==\"', '==', thestr)
-                if thestr not in theproperty:
-                    theproperty.append(thestr)
-                    analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
-                    analysis_file.write("- "+thestr+"\n")
-                    analysis_file.close()
-                    if self.verbose:
-                        print("Base64 string: ", thestr)
+                thestr = re.sub('"','', thestr)
+                    
+                # try to decode the Base64 string
+                try:
+                    decoded = base64.b64decode(thestr)
+                    
+                    if all(c in printable_chars for c in decoded) and decoded != b'' and (not decoded.decode('utf-8') in theproperty):
+                        # the decoded string is printable, so likely a valid decoded base64 info
+                        # + the array contains unique strings only
+                        theproperty.append(decoded.decode('utf-8'))
+                        if self.verbose:
+                            print("Base64 string: ", decoded)
+                except Exception as e:
+                    # this is not a base64 string
+                    pass
 
+            if len(theproperty) > 0:
+                analysis_file = open(os.path.join(self.outdir, droidlysis3.property_dump_file), 'a')
+                analysis_file.write("Base64 strings:\n")
 
-
-            
-            
-
-
-        
-
-
-
-                
-                                                
-                                                
-
+                for b in theproperty:
+                    analysis_file.write("- "+b+"\n")
+                    
+                analysis_file.close()
 
             
